#!/bin/bash

if [ $# -lt 2 ]; then
	echo -e "Generates type-specific list headers and implementations for the specified types."
	echo -e ""
	echo -e "Usage:"
	echo -e "\t$0 outputDirectory VuoTypeName:isReferenceCounted VuoTypeName:isReferenceCounted ..."
	echo -e ""
	echo -e "Prints the names of the created source file(s)."
	exit 1
fi

outputDirectory=$1
shift

for kv ; do
	vuoTypeName="${kv%%:*}"
	isReferenceCounted="${kv##*:}"
	typeSpecificH="$outputDirectory/VuoList_$vuoTypeName.h"
	typeSpecificCC="$outputDirectory/VuoList_$vuoTypeName.cc"

	if [ ! -f $typeSpecificH -o VuoList.h -nt $typeSpecificH ]; then
		sed -e "s/#error.*/\/\/ This header is generated by vuo\/type\/list\/generateVariants.sh./" \
			-e "s/LIST_TYPE/VuoList_$vuoTypeName/g" \
			-e "s/ELEMENT_TYPE/$vuoTypeName/g" \
			< VuoList.h \
			> $typeSpecificH
	fi

	if [ ! -f $typeSpecificCC -o VuoList.cc -nt $typeSpecificCC ]; then
		sed -e "s/#error.*/\/\/ This module is generated by vuo\/type\/list\/generateVariants.sh./" \
			-e "s/LIST_TYPE/VuoList_$vuoTypeName/g" \
			-e "s/ELEMENT_TYPE/$vuoTypeName/g" \
			-e "s/IS_ELEMENT_REFERENCE_COUNTED/$isReferenceCounted/g" \
			< VuoList.cc \
			> $typeSpecificCC
	fi

	echo -n "VuoList_$vuoTypeName.cc;"
done
